@@ -2,10 +2,9 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2013-2017 CERN
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2013-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
- * Copyright (C) 2013-2021 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -205,7 +204,7 @@ void SHAPE_LINE_CHAIN::amendArc( size_t aArcIndex, const VECTOR2I& aNewStart,
                                  const VECTOR2I& aNewEnd )
 {
     wxCHECK_MSG( aArcIndex <  m_arcs.size(), /* void */,
-                 "Invalid arc index requested." );
+                 wxT( "Invalid arc index requested." ) );
 
     SHAPE_ARC& theArc = m_arcs[aArcIndex];
 
@@ -230,7 +229,7 @@ void SHAPE_LINE_CHAIN::splitArc( ssize_t aPtIndex, bool aCoincident )
         return; // Nothing to do
 
     wxCHECK_MSG( aPtIndex < static_cast<ssize_t>( m_shapes.size() ), /* void */,
-                 "Invalid point index requested." );
+                 wxT( "Invalid point index requested." ) );
 
     if( IsSharedPt( aPtIndex ) || IsArcEnd( aPtIndex ) )
     {
@@ -411,7 +410,7 @@ bool SHAPE_LINE_CHAIN::Collide( const VECTOR2I& aP, int aClearance, int* aActual
         const SHAPE_ARC& arc = Arc( i );
 
         // The arcs in the chain should have zero width
-        wxASSERT_MSG( arc.GetWidth() == 0, "Invalid arc width - should be zero" );
+        wxASSERT_MSG( arc.GetWidth() == 0, wxT( "Invalid arc width - should be zero" ) );
 
         if( arc.Collide( aP, aClearance, aActual, aLocation ) )
             return true;
@@ -549,7 +548,7 @@ bool SHAPE_LINE_CHAIN::Collide( const SEG& aSeg, int aClearance, int* aActual,
         const SHAPE_ARC& arc = Arc( i );
 
         // The arcs in the chain should have zero width
-        wxASSERT_MSG( arc.GetWidth() == 0, "Invalid arc width - should be zero" );
+        wxASSERT_MSG( arc.GetWidth() == 0, wxT( "Invalid arc width - should be zero" ) );
 
         if( arc.Collide( aSeg, aClearance, aActual, aLocation ) )
             return true;
@@ -1167,7 +1166,8 @@ const SHAPE_LINE_CHAIN SHAPE_LINE_CHAIN::Slice( int aStartIndex, int aEndIndex )
         }
         else
         {
-            wxASSERT_MSG( !IsArcSegment( i ), "Still on an arc segment, we missed something..." );
+            wxASSERT_MSG( !IsArcSegment( i ),
+                          wxT( "Still on an arc segment, we missed something..." ) );
 
             rv.Append( m_points[i] );
         }
@@ -1505,8 +1505,6 @@ int SHAPE_LINE_CHAIN::PathLength( const VECTOR2I& aP, int aIndex ) const
     for( int i = 0; i < SegmentCount(); i++ )
     {
         const SEG seg = CSegment( i );
-        int d = seg.Distance( aP );
-
         bool indexMatch = true;
 
         if( aIndex >= 0 )
@@ -1745,46 +1743,36 @@ SHAPE_LINE_CHAIN& SHAPE_LINE_CHAIN::Simplify( bool aRemoveColinear )
     i = 0;
 
     // stage 2: eliminate colinear segments
-    while( i < np - 2 )
+    for( i = 1 ; i < np - 1 ; i ++ )
     {
-        const VECTOR2I p0 = pts_unique[i];
-        const VECTOR2I p1 = pts_unique[i + 1];
-        int n = i;
+        const VECTOR2I p_prev = pts_unique[i - 1];
+        const VECTOR2I midpoint = pts_unique[i];
+        const VECTOR2I p_next = pts_unique[i + 1];
 
-        if( aRemoveColinear && shapes_unique[i] == SHAPES_ARE_PT
-            && shapes_unique[i + 1] == SHAPES_ARE_PT )
+
+        if( aRemoveColinear && shapes_unique[i - 1] == SHAPES_ARE_PT
+            && shapes_unique[i] == SHAPES_ARE_PT )
         {
-            while( n < np - 2
-                    && ( SEG( p0, p1 ).LineDistance( pts_unique[n + 2] ) <= 1
-                            || SEG( p0, p1 ).Collinear( SEG( p1, pts_unique[n + 2] ) ) ) )
-                n++;
-        }
+            const auto distToMidpoint = SEG( p_prev, p_next ).LineDistance( midpoint );
+            const auto isMidpointColinear = SEG( p_prev, p_next ).Collinear( SEG( p_prev, midpoint ) );
 
-        m_points.push_back( p0 );
+            if( distToMidpoint <= 1 || isMidpointColinear )
+            {
+                pts_unique.erase( pts_unique.begin() +  i );
+                shapes_unique.erase( shapes_unique.begin() + i );
+                i--;
+                np--;
+                continue;
+            }
+        }
+    }
+
+    for( i = 0 ; i < np; i ++ )
+    {
+        m_points.push_back( pts_unique[i] );
         m_shapes.push_back( shapes_unique[i] );
-
-        if( n > i )
-            i = n;
-
-        if( n == np - 2 )
-        {
-            m_points.push_back( pts_unique[np - 1] );
-            m_shapes.push_back( shapes_unique[np - 1] );
-            return *this;
-        }
-
-        i++;
     }
 
-    if( np > 1 )
-    {
-        m_points.push_back( pts_unique[np - 2] );
-        m_shapes.push_back( shapes_unique[np - 2] );
-    }
-
-    m_points.push_back( pts_unique[np - 1] );
-    m_shapes.push_back( shapes_unique[np - 1] );
-
     assert( m_points.size() == m_shapes.size() );
 
     return *this;
