@@ -71,7 +71,8 @@ ZONE_FILLER::~ZONE_FILLER()
 void ZONE_FILLER::SetProgressReporter( PROGRESS_REPORTER* aReporter )
 {
     m_progressReporter = aReporter;
-    wxASSERT_MSG( m_commit, "ZONE_FILLER must have a valid commit to call SetProgressReporter" );
+    wxASSERT_MSG( m_commit, wxT( "ZONE_FILLER must have a valid commit to call "
+                                 "SetProgressReporter" ) );
 }
 
 
@@ -362,10 +363,16 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
         }
     }
 
-    // Now remove islands outside the board edge
+    // Now remove islands which are either outside the board edge or fail to meet the minimum
+    // area requirements
     for( ZONE* zone : aZones )
     {
-        LSET zoneCopperLayers = zone->GetLayerSet() & LSET::AllCuMask( MAX_CU_LAYERS );
+        LSET   zoneCopperLayers = zone->GetLayerSet() & LSET::AllCuMask( MAX_CU_LAYERS );
+
+        // Min-thickness is the web thickness.  On the other hand, a blob min-thickness by
+        // min-thickness is not useful.  Since there's no obvious definition of web vs. blob, we
+        // arbitrarily choose "at least 2X the area".
+        double minArea = (double) zone->GetMinThickness() * zone->GetMinThickness() * 2;
 
         for( PCB_LAYER_ID layer : zoneCopperLayers.Seq() )
         {
@@ -378,8 +385,12 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
             {
                 std::vector<SHAPE_LINE_CHAIN>& island = poly.Polygon( ii );
 
-                if( island.empty() || !m_boardOutline.Contains( island.front().CPoint( 0 ) ) )
+                if( island.empty()
+                        || !m_boardOutline.Contains( island.front().CPoint( 0 ) )
+                        || island.front().Area() < minArea )
+                {
                     poly.DeletePolygon( ii );
+                }
             }
 
             zone->SetFilledPolysList( layer, poly );
@@ -592,6 +603,7 @@ void ZONE_FILLER::addKnockout( BOARD_ITEM* aItem, PCB_LAYER_ID aLayer, int aGap,
     case PCB_SHAPE_T:
     case PCB_TEXT_T:
     case PCB_FP_SHAPE_T:
+    case PCB_TARGET_T:
         aItem->TransformShapeWithClearanceToPolygon( aHoles, aLayer, aGap, m_maxError,
                                                      ERROR_OUTSIDE, aIgnoreLineWidth );
         break;
@@ -707,10 +719,8 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
 
                     if( aPad->GetNetCode() > 0 && aPad->GetNetCode() == aZone->GetNetCode() )
                     {
-                        // For pads having the same netcode as the zone, the net and hole
-                        // clearances have no meanings.
-                        // So just knock out the greater of the zone's local clearance and
-                        // thermal relief.
+                        // For unconnected but same-net pads, electrical clearances don't apply.
+                        // Use the greater of the zone's local clearance and thermal relief.
                         gap = std::max( zone_clearance, aZone->GetThermalReliefGap( aPad ) );
                         knockoutHoleClearance = false;
                     }
@@ -765,18 +775,7 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                         int      gap = 0;
                         bool     checkHoleClearance = true;
 
-                        if( via->GetNetCode() > 0 && via->GetNetCode() == aZone->GetNetCode() )
-                        {
-                            // For pads having the same netcode as the zone, the net and hole
-                            // clearances have no meanings.
-                            // So just knock out the zone's local clearance.
-                            gap = zone_clearance;
-                            checkHoleClearance = false;
-                        }
-                        else
-                        {
-                            gap = evalRulesForItems( CLEARANCE_CONSTRAINT, aZone, aTrack, aLayer );
-                        }
+                        gap = evalRulesForItems( CLEARANCE_CONSTRAINT, aZone, aTrack, aLayer );
 
                         if( via->FlashLayer( aLayer ) )
                         {
@@ -813,7 +812,7 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
         if( !track->IsOnLayer( aLayer ) )
             continue;
 
-        if( track->GetNetCode() == aZone->GetNetCode()  && ( aZone->GetNetCode() != 0) )
+        if( track->GetNetCode() == aZone->GetNetCode() && ( aZone->GetNetCode() != 0) )
             continue;
 
         if( checkForCancel( m_progressReporter ) )
@@ -853,7 +852,7 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                                                                     aZone, aItem, Margin ) );
                         }
 
-                        addKnockout( aItem, aLayer, gap, ignoreLineWidths, aHoles );
+                        addKnockout( aItem, aLayer, gap + extra_margin, ignoreLineWidths, aHoles );
                     }
                 }
             };
@@ -865,8 +864,7 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
         knockoutGraphicClearance( &footprint->Reference() );
         knockoutGraphicClearance( &footprint->Value() );
 
-        // Don't knock out holes in zones that share a net
-        // with a nettie footprint
+        // Don't knock out holes in zones that share a net with a nettie footprint
         if( footprint->IsNetTie() )
         {
             for( PAD* pad : footprint->Pads() )
@@ -913,7 +911,8 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                     if( aKnockout->GetIsRuleArea() )
                     {
                         // Keepouts use outline with no clearance
-                        aKnockout->TransformSmoothedOutlineToPolygon( aHoles, 0, nullptr );
+                        aKnockout->TransformSmoothedOutlineToPolygon( aHoles, 0, m_maxError,
+                                                                      ERROR_OUTSIDE, nullptr );
                     }
                     else if( bds.m_ZoneFillVersion == 5 )
                     {
@@ -921,7 +920,8 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                         int gap = evalRulesForItems( CLEARANCE_CONSTRAINT, aZone, aKnockout,
                                                      aLayer );
 
-                        aKnockout->TransformSmoothedOutlineToPolygon( aHoles, gap, nullptr );
+                        aKnockout->TransformSmoothedOutlineToPolygon( aHoles, gap, m_maxError,
+                                                                      ERROR_OUTSIDE, nullptr );
                     }
                     else
                     {
@@ -930,7 +930,8 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
                                                      aLayer );
 
                         SHAPE_POLY_SET poly;
-                        aKnockout->TransformShapeWithClearanceToPolygon( poly, aLayer, gap,
+                        aKnockout->TransformShapeWithClearanceToPolygon( poly, aLayer,
+                                                                         gap + extra_margin,
                                                                          m_maxError,
                                                                          ERROR_OUTSIDE );
                         aHoles.Append( poly );
@@ -1001,7 +1002,13 @@ void ZONE_FILLER::subtractHigherPriorityZones( const ZONE* aZone, PCB_LAYER_ID a
 
                 if( aKnockout->GetCachedBoundingBox().Intersects( aZone->GetCachedBoundingBox() ) )
                 {
-                    aRawFill.BooleanSubtract( *aKnockout->Outline(), SHAPE_POLY_SET::PM_FAST );
+                    // Processing of arc shapes in zones is not yet supported because Clipper
+                    // can't do boolean operations on them.  The poly outline must be converted to
+                    // segments first.
+                    SHAPE_POLY_SET outline = *aKnockout->Outline();
+                    outline.ClearArcs();
+
+                    aRawFill.BooleanSubtract( outline, SHAPE_POLY_SET::PM_FAST );
                 }
             };
 
@@ -1084,19 +1091,19 @@ bool ZONE_FILLER::computeRawFilledArea( const ZONE* aZone,
     SHAPE_POLY_SET clearanceHoles;
 
     aRawPolys = aSmoothedOutline;
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In1_Cu, "smoothed-outline" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In1_Cu, wxT( "smoothed-outline" ) );
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
         return false;
 
     knockoutThermalReliefs( aZone, aLayer, aRawPolys );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In2_Cu, "minus-thermal-reliefs" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In2_Cu, wxT( "minus-thermal-reliefs" ) );
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
         return false;
 
     buildCopperItemClearances( aZone, aLayer, clearanceHoles );
-    DUMP_POLYS_TO_COPPER_LAYER( clearanceHoles, In3_Cu, "clearance-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( clearanceHoles, In3_Cu, wxT( "clearance-holes" ) );
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
         return false;
@@ -1111,16 +1118,16 @@ bool ZONE_FILLER::computeRawFilledArea( const ZONE* aZone,
     static const bool USE_BBOX_CACHES = true;
     SHAPE_POLY_SET testAreas = aRawPolys;
     testAreas.BooleanSubtract( clearanceHoles, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( testAreas, In4_Cu, "minus-clearance-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( testAreas, In4_Cu, wxT( "minus-clearance-holes" ) );
 
     // Prune features that don't meet minimum-width criteria
     if( half_min_width - epsilon > epsilon )
     {
         testAreas.Deflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
-        DUMP_POLYS_TO_COPPER_LAYER( testAreas, In5_Cu, "spoke-test-deflated" );
+        DUMP_POLYS_TO_COPPER_LAYER( testAreas, In5_Cu, wxT( "spoke-test-deflated" ) );
 
         testAreas.Inflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
-        DUMP_POLYS_TO_COPPER_LAYER( testAreas, In6_Cu, "spoke-test-reinflated" );
+        DUMP_POLYS_TO_COPPER_LAYER( testAreas, In6_Cu, wxT( "spoke-test-reinflated" ) );
     }
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
@@ -1169,19 +1176,31 @@ bool ZONE_FILLER::computeRawFilledArea( const ZONE* aZone,
         }
     }
 
-    DUMP_POLYS_TO_COPPER_LAYER( debugSpokes, In7_Cu, "spokes" );
+    DUMP_POLYS_TO_COPPER_LAYER( debugSpokes, In7_Cu, wxT( "spokes" ) );
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
         return false;
 
     aRawPolys.BooleanSubtract( clearanceHoles, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In8_Cu, "after-spoke-trimming" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In8_Cu, wxT( "after-spoke-trimming" ) );
 
     // Prune features that don't meet minimum-width criteria
     if( half_min_width - epsilon > epsilon )
         aRawPolys.Deflate( half_min_width - epsilon, numSegs, cornerStrategy );
 
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In9_Cu, "deflated" );
+    // Min-thickness is the web thickness.  On the other hand, a blob min-thickness by
+    // min-thickness is not useful.  Since there's no obvious definition of web vs. blob, we
+    // arbitrarily choose "at least 1/2 min-thickness on one axis".
+    for( int ii = aRawPolys.OutlineCount() - 1; ii >= 0; ii-- )
+    {
+        std::vector<SHAPE_LINE_CHAIN>& island = aRawPolys.Polygon( ii );
+        EDA_RECT                       islandExtents = island.front().BBox();
+
+        if( islandExtents.GetSizeMax() < half_min_width )
+            aRawPolys.DeletePolygon( ii );
+    }
+
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In9_Cu, wxT( "deflated" ) );
 
     if( m_progressReporter && m_progressReporter->IsCancelled() )
         return false;
@@ -1207,18 +1226,18 @@ bool ZONE_FILLER::computeRawFilledArea( const ZONE* aZone,
         aRawPolys.Inflate( half_min_width - epsilon, numSegs, cornerStrategy );
     }
 
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In15_Cu, "after-reinflating" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In15_Cu, wxT( "after-reinflating" ) );
 
     // Ensure additive changes (thermal stubs and particularly inflating acute corners) do not
     // add copper outside the zone boundary or inside the clearance holes
     aRawPolys.BooleanIntersection( aMaxExtents, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In16_Cu, "after-trim-to-outline" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In16_Cu, wxT( "after-trim-to-outline" ) );
     aRawPolys.BooleanSubtract( clearanceHoles, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In17_Cu, "after-trim-to-clearance-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In17_Cu, wxT( "after-trim-to-clearance-holes" ) );
 
     // Lastly give any same-net but higher-priority zones control over their own area.
     subtractHigherPriorityZones( aZone, aLayer, aRawPolys );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In18_Cu, "minus-higher-priority-zones" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In18_Cu, wxT( "minus-higher-priority-zones" ) );
 
     aRawPolys.Fracture( SHAPE_POLY_SET::PM_FAST );
     return true;
@@ -1363,7 +1382,7 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
 
             // For circle pads, the thermal spoke orientation is 45 deg
             if( pad->GetShape() == PAD_SHAPE::CIRCLE )
-                padAngle = s_RoundPadThermalSpokeAngle;
+                padAngle += s_RoundPadThermalSpokeAngle;
 
             for( int i = 0; i < 4; i++ )
             {
@@ -1545,7 +1564,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
     if( orientation != 0.0 )
         holes.Rotate( -M_PI/180.0 * orientation, VECTOR2I( 0,0 ) );
 
-    DUMP_POLYS_TO_COPPER_LAYER( holes, In10_Cu, "hatch-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( holes, In10_Cu, wxT( "hatch-holes" ) );
 
     int outline_margin = aZone->GetMinThickness() * 1.1;
 
@@ -1558,12 +1577,12 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
     SHAPE_POLY_SET deflatedFilledPolys = aRawPolys;
     deflatedFilledPolys.Deflate( outline_margin - aZone->GetMinThickness(), 16 );
     holes.BooleanIntersection( deflatedFilledPolys, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( holes, In11_Cu, "fill-clipped-hatch-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( holes, In11_Cu, wxT( "fill-clipped-hatch-holes" ) );
 
     SHAPE_POLY_SET deflatedOutline = *aZone->Outline();
     deflatedOutline.Deflate( outline_margin, 16 );
     holes.BooleanIntersection( deflatedOutline, SHAPE_POLY_SET::PM_FAST );
-    DUMP_POLYS_TO_COPPER_LAYER( holes, In12_Cu, "outline-clipped-hatch-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( holes, In12_Cu, wxT( "outline-clipped-hatch-holes" ) );
 
     if( aZone->GetNetCode() != 0 )
     {
@@ -1622,7 +1641,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
 
         holes.BooleanSubtract( aprons, SHAPE_POLY_SET::PM_FAST );
     }
-    DUMP_POLYS_TO_COPPER_LAYER( holes, In13_Cu, "pad-via-clipped-hatch-holes" );
+    DUMP_POLYS_TO_COPPER_LAYER( holes, In13_Cu, wxT( "pad-via-clipped-hatch-holes" ) );
 
     // Now filter truncated holes to avoid small holes in pattern
     // It happens for holes near the zone outline
@@ -1639,7 +1658,7 @@ bool ZONE_FILLER::addHatchFillTypeOnZone( const ZONE* aZone, PCB_LAYER_ID aLayer
     // create grid. Use SHAPE_POLY_SET::PM_STRICTLY_SIMPLE to
     // generate strictly simple polygons needed by Gerber files and Fracture()
     aRawPolys.BooleanSubtract( aRawPolys, holes, SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
-    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In14_Cu, "after-hatching" );
+    DUMP_POLYS_TO_COPPER_LAYER( aRawPolys, In14_Cu, wxT( "after-hatching" ) );
 
     return true;
 }
